<?php

class BaseLocationAutocomplete
{
    public static function find($country, $q)
    {
        $results = array();
        $q = self::fixTerm($q);
        $results['location_name'] = $q;

        $popular = self::findPopular($country, $q);

        $results[] = array('text' => 'Any', 'id' => '');

        if (!empty($popular))
        {
            $results[] = array(
                'text' => 'Popular',
                'children' => $popular
            );
        }

        if ($q)
        {
            $results = array_merge($results, self::findLocations($country, $q));
        }

        return $results;
    }

    private static function findPopular($country, $q)
    {
        $popularList = LocationDeparture::departureList($country, $q);
        $popular = array();

        foreach ($popularList as $key => $value)
        {
          $popular[] = array('id' => 'departure:'.$key, 'text' => $value);
        }

        return $popular;
    }

    protected static function findLocations($country, $q, $prefix = 'location:', $allowAny = true)
    {
        $results = array();
        $categories = Doctrine::getTable('LocationCategory')->findAllWithLocations($country, $q);

        foreach ($categories as $category)
        {
            $cities = $allowAny ? array(array(
                'id' => 'category:'.$category->getId(),
                'text' => 'Any from '.$category->get('description')
            )) : array();

            foreach ($category->get('Locations') as $location)
            {
                $cities[] = array(
                    'id' => $prefix.$location->getId(),
                    'text' => $location->get('name')
                );
            }

            if (!empty($cities))
            {
                $results[] = array(
                'text' => $category->get('description'),
                'children' => $cities
                );
            }
        }

        return $results;
    }

    protected static function fixTerm($q)
    {
        if (strpos($q, ':') !== false)
        {
            list($type, $id) = explode(':', $q);

            switch ($type) {
                case 'departure':
                    $departure = Doctrine::getTable('LocationDeparture')->find($id);
                    $q = $departure ? $departure->get('name') : $q;
                    break;

                case 'category':
                    $category = Doctrine::getTable('LocationCategory')->find($id);
                    $q = $category ? $category->get('description') : $q;

                    break;
                case 'location':
                    $category = Doctrine::getTable('Locations')->find($id);

                    $q = $category ? $category->get('name') : $q;
                    break;
            }
        }

        return $q;
    }
}