<?php

class tcOwnerUser extends sfBasicSecurityUser
{
    public function getProfile()
    {
        return $this->getAttribute('profile');
    }

    public function getOwnerId()
    {
        $field = $this->getAttribute('profile') instanceof User ? 'id' : 'owner_id';
        return $this->getAttribute('profile')->get($field); // TODO: use owner_id field
    }

    public function getId()
    {
        return $this->isAuthenticated() ? $this->getProfile()->get('id') : -1;
    }

    public function getCountry()
    {
        return $this->getAttribute('country', 1);
    }

    public function getCountryName()
    {
        $country = Doctrine::getTable('LocationCountry')->find($this->getCountry());

        return $country->get('name');
    }

    public function setCountry($country)
    {
        return $this->setAttribute('country', $country);
    }

    public function getUserDomain()
    {
        switch($this->getCountry())
        {
            default:
            case 1:
                return 'nz';
            case 2:
                return 'au';
            case 3:
                return 'us';
        }
    }

    public function getAttribute($name, $default = null, $ns = null)
    {
        if ('country' == $name && null === $default) {
            return $this->getCountry();
        }

        return parent::getAttribute($name, $default, $ns);
    }

    public function authenticate($user)
    {
        $this->setAuthenticated(true);
        $this->setAttribute('profile', $user);
        $this->dispatcher->notify(new sfEvent($user, 'manager.logged_in'));

        if ($user instanceof OwnerUser) {
            $this->addCredentials('IS_OWNER_USER');
            if ($user->get('is_admin')) {
                $this->addCredentials('IS_OWNER');
            }

            $this->setCountry($user->get('country_id'));
            $this->setCulture('en_'.strtoupper($this->getUserDomain()));
            $user->get('Owner')->set('last_login', date('Y-m-d H:i:s'));
            $user->get('Owner')->save();
        } else {
            $this->addCredentials('IS_OWNER');
            $user->set('last_login', date('Y-m-d H:i:s'));
            $user->save();
        }
    }

    public function impersonate($user, $adminUser)
    {
        $this->authenticate($user);
        $this->addCredentials('IS_IMPERSONATED');
        $this->setAttribute('admin_id', $adminUser->getId());
        $this->setAttribute('admin_name', $adminUser->get('name'));
    }


    public function pushFlash($name, $value)
    {
      if (!($values = $this->getFlash($name)))
      {
        $values = array();
      }

      $values[] = $value;

      return $this->setFlash($name, $values);
    }
}
