<?php


class dashboardActions extends sfActions
{
  public function executeIndex() 
  {
  }

  public function executeApi($request)
  {
    $conn = Doctrine_Manager::getInstance()->getCurrentConnection()->getDbh();
    $fetcher = new ROApi($conn, $this->getUser()->getCountry());

    $ro = $this->getUser()->getOwnerId();
    $listings = $fetcher->getListings($ro, $request->getParameter('from_date'), $request->getParameter('to_date'), $request->getParameter('pickup'), $request->getParameter('dropoff'));
    $bookings = $fetcher->getBookings($ro,$request->getParameter('from_date'), $request->getParameter('to_date'), $request->getParameter('pickup'), $request->getParameter('dropoff'));
    $routes = $fetcher->getRoutes($ro, $request->getParameter('from_date'), $request->getParameter('to_date'), $request->getParameter('pickup'), $request->getParameter('dropoff'));
    $nps = $fetcher->getNPS($ro, $request->getParameter('from_date'), $request->getParameter('to_date'));

    $data = [
      'listings' => $listings,
      'bookings' => $bookings,
      'routes' => $routes,
      'nps' => $nps
    ];

    if ($request->getParameter('export') && in_array($request->getParameter('export'), array_keys($data))) {
      $this->getResponse()->setHttpHeader('Content-Type', 'application/csv');
      $this->getResponse()->setHttpHeader('Content-Disposition', 'inline; filename=transfercar-export-' . date ( "Y-m-d" ) . '.csv;');
      $this->getResponse()->setHttpHeader('Cache-Control', 'private');

      return $this->renderText($this->exportCSV($data[$request->getParameter('export')]));
    }
    return $this->renderText(json_encode($data));
  }



  public function executeExportXlsx($request)
  {
    $spreadsheet = \PhpOffice\PhpSpreadsheet\IOFactory::load(sfConfig::get('sf_data_dir').'/template.xlsx');

    $worksheet = $spreadsheet->getActiveSheet();

    $csv_data = base64_decode($request->getParameter('csv_data'));
    $data = array_map('str_getcsv', explode("\n", $csv_data));
    array_walk($data[0], function(&$item) {
      $item = ucfirst($item);
    });
    $spreadsheet->getActiveSheet()
      ->fromArray(
          $data,  // The data to set
          NULL,        // Array values with this value will not be set
          'A3'         // Top left coordinate of the worksheet range where
                       //    we want to set these values (default is A1)
      );

      $roName = $this->getUser()->getAttribute('profile')->get('company');
      $title = sprintf('%s for %s. Insights provided by Transfercar Limited.', $request->getParameter('title', 'Report'), $roName);
      
      $cols = count($data[0])-1;
      $rows = count($data)-2; // total - heading

      $spreadsheet->getActiveSheet()->getStyle('A3:'.chr(65+$cols).'3')->applyFromArray([
        'font' => [
            'bold' => true,
        ],
        'borders' => [
          'outline' => [
            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THICK,
          ]
        ]
      ]);
    // chr(65) = A
      $spreadsheet->getActiveSheet()->getStyle('A4:'.chr(65+$cols).($rows+4))->applyFromArray([
        'borders' => [
          'outline' => [
            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THICK,
          ]
        ]
      ]);

      $spreadsheet->getActiveSheet()->getStyle('A4:A'.($rows+4))->applyFromArray([
        'font' => [
            'bold' => true,
        ],
        'borders' => [
          'right' => [
            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THICK,
          ],
        ]
      ]);

      for ($l = 65; $l < 65+$cols;$l++){
        $spreadsheet->getActiveSheet()->getColumnDimension(chr($l))->setAutoSize(true);
      }

      if (preg_match('/cost saving/i', $title)) {
        $spreadsheet->getActiveSheet()->getStyle('C4:E'.($rows+4))->getNumberFormat()->setFormatCode(\PhpOffice\PhpSpreadsheet\Style\NumberFormat::FORMAT_CURRENCY_USD_SIMPLE);
      } else {

        $spreadsheet->getActiveSheet()->getStyle(chr(63+$cols).'4:'.chr(65+$cols).($rows+4))->getNumberFormat()->setFormatCode(\PhpOffice\PhpSpreadsheet\Style\NumberFormat::FORMAT_PERCENTAGE_00); 
      }

      $spreadsheet->getActiveSheet()->setCellValue('A1', $title);

      $writer = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, 'Xls');


      $name = strtolower(str_replace(' ', '-', $request->getParameter('title', 'report').' '.$roName.' '.date('Y-m-d-His'))).'.xls';

      // $filename = sfConfig::get('sf_upload_dir').'/transfercar-export-'.$name;
      // $filename = pathinfo($filePath, PATHINFO_BASENAME);
      $this->getResponse()->setHttpHeader("Content-type", "application/octet-stream");
      $this->getResponse()->setHttpHeader("Accept-Ranges", "bytes");
      $this->getResponse()->setHttpHeader("Content-type", "application/vnd.ms-excel");
      $this->getResponse()->setHttpHeader("Content-Disposition", "attachment;filename=transfercar-export-".$name);
      $this->getResponse()->setHttpHeader("Pragma", " no-cache");
      $this->getResponse()->setHttpHeader("Expires", " 0");
      $this->getResponse()->sendHttpHeaders();
      $writer->save('php://output');
      exit;
      // $this->downloadFile($filename);
  }

  private function downloadFile($filePath)
  {
      $filename = pathinfo($filePath, PATHINFO_BASENAME);
      $this->getResponse()->setHttpHeader("Content-type", "application/octet-stream");
      $this->getResponse()->setHttpHeader("Accept-Ranges", "bytes");
      $this->getResponse()->setHttpHeader("Content-type", "application/vnd.ms-excel");
      $this->getResponse()->setHttpHeader("Content-Disposition", "attachment;filename=".$filename);
      $this->getResponse()->setHttpHeader("Pragma", " no-cache");
      $this->getResponse()->setHttpHeader("Expires", " 0");

      $this->getResponse()->sendHttpHeaders();

      $out = fopen('php://output', 'w');
      $file = fopen($filePath, 'r');

      stream_copy_to_stream($file, $out);

      fclose($out);
      fclose($file);
      error_log('unlink '.$filePath);
      unlink($filePath);
      exit;
  }

  private function exportCSV($csv)
  {
      $stream = fopen('php://temp', 'w');

      fputcsv($stream, array_keys($csv));
      foreach ($csv as $line) {
          fputcsv($stream, $line);
      }
      rewind($stream);
      $content = stream_get_contents($stream);
      fclose($stream);

      return $content;
  }
}
