<?php

use Transfercar\Geoloc\GoogleMapsDistancesMatrix;

/**
 * listing actions.
 *
 * @package    symfony
 * @subpackage listing
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 5647 2007-10-23 17:25:20Z Eric.Fredj $
 */
class listingActions extends autolistingActions
{
    public function preExecute ()
    {
      if (!in_array($this->getRequestParameter('action'), array('list', 'edit', 'create')))
      {
        $this->getResponse()->addStylesheet('/sf/sf_admin/css/main');
      }
    }

    public function executeView(sfWebRequest $request)
    {
      $this->listing = $this->getListingOrCreate();
      $this->forward404Unless($this->listing->exists());
    }

    public function executeImport(sfWebRequest $request)
    {
        $form = new ListingImportForm([], ['owner_id' => $this->getUser()->getOwnerId()]);
        if ($request->isMethod('POST')) {
            $form->bind($request->getParameter('import'), $request->getFiles('import'));

            if ($form->isValid()) {
                $import = new Transfercar\Import\ListingImport();

                try {
                  $import->import(
                    $this->getUser()->getProfile(), 
                    $form->getValue('import')
                  );

                  $this->getUser()->setFlash('notice', 'Your listings have been imported');
                  return $this->redirect('listing/index');
                } catch (\Exception $e) {
                  $this->error = $e->getMessage();
                }
            }
        }

        $this->form = $form;
    }

    public function executeEdit(sfWebRequest $request = null)
    {
        $listing = $this->getListingOrCreate();
        $listing->set('owner_id', $this->getUser()->getOwnerId());

        $this->forward404Unless((
          !$listing->exists() ||
          $listing->isStatus(array(
            Listing::STATUS_PAUSED,
            Listing::STATUS_EXPIRED,
            Listing::STATUS_CANCELED
          )) ||
          ($listing->getAcceptedRequest()->exists() && $listing->getAcceptedRequest()->isCancelled())
        ));

        if ($listing->hasPendingRequests())
        {
          $this->getUser()->setFlash('notice', 'A user requested this listing, you are not allowed to modify it anymore');

          return $this->redirect('listing/index');
        }

        $this->form = new OwnerListingForm(array(), array(
          'country' => $this->getUser()->getCountry(),
          'domain'  => $this->getUser()->getUserDomain(),
          'user'    => $this->getUser()->getAttribute('profile')
        ));

        $this->form->setDefaultsFromListing($listing);
        $this->listing = $listing;

        if ($request->isMethod('POST'))
        {
          $this->form->bind($request->getParameter('listing'));

          if ($this->form->isValid())
          {
            if ($this->getUser()->hasCredential('IS_IMPERSONATED'))
            {
              $listing->set('admin_id', $this->getUser()->getAttribute('admin_id'));
            }
            else
            {
              $listing->set('admin_id', null);
            }

            $this->form->save($listing);
            if ($request->getParameter('id')) {
              $this->getUser()->setFlash('notice', 'Your listing has been saved');
            } else {
              $this->getUser()->setFlash('notice', 'Your listing has been added');
              $this->dispatcher->notify(new sfEvent($listing, 'ro.listing.created'));
            }

            return $this->redirect('listing/index');
          }
        }
    }

    public function executeRate(sfWebRequest $request)
    {
        $this->listing = $this->getListingOrCreate();

        $this->forward404Unless($this->listing->isStatus(array(
          Listing::STATUS_ACCEPTED,
          Listing::STATUS_PICKED_UP,
          Listing::STATUS_DROPPED_OFF,
          Listing::STATUS_CANCELED
        )));

        $this->form = new RequestRateForm();

        if ($request->isMethod('POST'))
        {
            $this->form->bind($request->getParameter('request'));

            if ($this->form->isValid())
            {
                $req = $this->listing->getAcceptedRequest();
                $values = $this->form->getValues();
                $req->rate($values['driver_rate'], $values['comment'], $values['internal_comment']);
                $req->close();

                $this->redirectUnless($request->isXmlHttpRequest(), 'listing/closed?id='.$this->listing->getId());

                $this->setVar('request', $req);
                $this->setTemplate('accepted', 'request');
            }
        }
    }

    public function executeCancel(sfWebRequest $request)
    {
      $this->listing = $this->getListingOrCreate();

      if ($this->listing->getAcceptedRequest())
      {
        if ($this->listing->getAcceptedRequest()->pickupInLessThan('+2 days'))
        {
          $this->setTemplate('lateCancel');
        }
      }

      $this->form = new RequestCancelledByOwnerForm();

      if ($request->isMethod('POST'))
      {
        $this->form->bind($request->getParameter('request'));

        if ($this->form->isValid())
        {
          $req = $this->listing->getAcceptedRequest();
          $values = $this->form->getValues();
          $req->set('cancellation_reason_id', $values['reason']);
          $req->set('is_withdrawn', Doctrine::getTable('CancellationReason')->isWithdrawn($req->get('cancellation_reason_id')));
          $req->cancel(
            Request::STATUS_OWNER_CANCELED,
            $this->listing->getAcceptedRequest()->pickupInLessThan('+2 days'),
            $values['comment'],
            $values['internal_comment']
          );

          $this->redirectUnless($request->isXmlHttpRequest(), 'listing/complete?id='.$this->listing->getId());

          $this->setVar('request', $req);
          $this->setTemplate('accepted', 'request');
        }
      }
    }

    public function executeDriverCancelled(sfWebRequest $request)
    {
      $this->listing = $this->getListingOrCreate();
      $this->form = new RequestCancelledByDriverForm();

       if ($request->isMethod('POST'))
       {
        $this->form->bind($request->getParameter('request'));

        if ($this->form->isValid())
        {
          $req = $this->listing->getAcceptedRequest();
          $values = $this->form->getValues();
          $req->set('cancellation_reason_id', $values['subcategory'] ? $values['subcategory'] : $values['category']);
          $req->set('is_withdrawn', Doctrine::getTable('CancellationReason')->isWithdrawn($req->get('cancellation_reason_id')));

          if ($values['canceltime'] == 'after_pickup') {
            $req->noshow($values['internal_comment']);
          } else {
            $req->cancel(Request::STATUS_DRIVER_CANCELED, $values['canceltime'] === 'less_48hours', $values['comment'], $values['internal_comment']);
          }

          $this->redirectUnless($request->isXmlHttpRequest(), 'listing/complete?id='.$this->listing->getId());

          $this->setVar('request', $req);
          $this->setTemplate('accepted', 'request');
        }
      }
    }

    public function executeNoshow(sfWebRequest $request)
    {
        $this->listing = $this->getListingOrCreate();
        $this->form = new RequestNoShowForm();
        unset($this->form['comment']);

        if ($request->isMethod('POST'))
        {
            $this->form->bind($request->getParameter('request'));

            if ($this->form->isValid())
            {
                $req = $this->listing->getAcceptedRequest();
                $values = $this->form->getValues();
                $req->noshow($values['internal_comment']);

                $this->redirectUnless($request->isXmlHttpRequest(), 'listing/closed?id='.$this->listing->getId());

                $this->setVar('request', $req);
                $this->setTemplate('accepted', 'request');
            }
        }
    }

    public function executeClosed()
    {
        $this->listing = $this->getListingOrCreate();
    }

    public function executePause()
    {
      $listing = $this->getListingOrCreate();

      $listing->pause();

      $this->getUser()->setFlash('notice', 'Listing has been paused');

      return $this->redirect('listing/index');
    }

    public function executeCopy()
    {
      $listing = $this->getListingOrCreate();
      $listing->set('owner_id', $this->getUser()->getId());

      $this->form = new OwnerListingForm(array(), array(
        'country' => $this->getUser()->getCountry(),
        'domain'  => $this->getUser()->getUserDomain(),
        'user'    => $this->getUser()->getAttribute('profile')
      ));

      $listing = $this->form->duplicateFromListing($listing);
      $this->listing = $listing;

      $this->setTemplate('edit');
    }

    public function executeConfirm($request)
    {
      $listing = $this->getListingOrCreate();

      $this->form = new ConfirmBookingForm(array(
        'internal_reference' => $listing->get('internal_reference')
      ));

      if ($request->isMethod('POST'))
      {
        $this->form->bind($request->getParameter('confirm'));

        if ($this->form->isValid())
        {
          $listing->confirmBooking($this->form->getValue('internal_reference'));

          return $this->renderPartial('global/close_modal');
        }
      }

      $this->listing = $listing;
    }

    /**
     * Show fee for relocation on given route
     * This action is called by "add listing" form observer
     *
     * @param sfWebRequest $request HTTP request object
     */
    public function executeShowFee($request)
    {
        //$this->setLayout(false);

        $from    = array_filter(explode('_', $request->getParameter('from')));
        $to      = array_filter(explode('_', $request->getParameter('to')));
        $vehicle = array_filter(explode('_', $request->getParameter('vehicle')));

        if (!$from || !$to || !$vehicle)
        {
          $this->price = '';
          return sfView::SUCCESS;
        }

        if ('owner' === $from[0]) {
          $branch = Doctrine::getTable('OwnerBranch')->find($from[1]);
          $from = $branch ? $branch->get('location_id') : 0;
        } else {
          $from = $from[1];
        }

        if ('owner' === $to[0]) {
          $branch = Doctrine::getTable('OwnerBranch')->find($to[1]);
          $to = $branch ? $branch->get('location_id') : 0;
        } else {
          $to = $to[1];
        }

        if ('owner' === $vehicle[0]) {
          $vehicle = Doctrine::getTable('OwnerVehicleType')->find($vehicle[1]);
          $vehicle = $vehicle ? $vehicle->get('vehicle_type_id') : 0;
        } else {
          $vehicle = $vehicle[1];
        }

        $price = Doctrine::getTable('Prices')->getPriceFor($this->getUser()->getOwnerId(), $from, $to, $vehicle);

        if(!$price){
            $this->price = '';
        } else {
            $this->price = $price . " " . sfConfig::get("sf_currency_new");
        }

        return sfView::SUCCESS;
    }

    public function executeShowAdditionalDaysFee(sfWebRequest $request)
    {
      $from    = array_filter(explode('_', $request->getParameter('from')));
      $to      = array_filter(explode('_', $request->getParameter('to')));
      $vehicle = array_filter(explode('_', $request->getParameter('vehicle_type_id')));

      if (!$from || !$to || !$vehicle)
      {
        $this->price = '';
        return $this->renderText('?');
      }

      if ('owner' === $from[0]) {
        $branch = Doctrine::getTable('OwnerBranch')->find($from[1]);
        $from = $branch ? $branch->get('NearestLocation') : 0;
      } else {
        $from = Doctrine::getTable('Locations')->find($from[1]);
      }

      if ('owner' === $to[0]) {
        $branch = Doctrine::getTable('OwnerBranch')->find($to[1]);
        $to = $branch ? $branch->get('NearestLocation') : 0;
      } else {
        $to = Doctrine::getTable('Locations')->find($to[1]);
      }

      if ('owner' === $vehicle[0]) {
        $vehicle = Doctrine::getTable('OwnerVehicleType')->find($vehicle[1]);
        $vehicle = $vehicle ? $vehicle->get('VehicleType') : 0;
      } else {
        $vehicle = Doctrine::getTable('Locations')->find($vehicle[1]);
      }

      if (!$from || !$to || !$vehicle)
      {
        $this->price = '';
        return $this->renderText('0');
      }

      $price = Doctrine::getTable('Prices')->getPriceForAdditionalDays($from, $to, $vehicle->getId(), $this->getUser()->getOwnerId());

      return $this->renderText($price);
    }

    public function executeCheckFreeDays(sfWebRequest $request)
    {
      $from     = $request->getParameter('from');
      $to       = $request->getParameter('to');
      $freeDays = intval($request->getParameter('nb_days'));


      $locationTable = Doctrine::getTable('Locations');

      $from = $locationTable->findFromBranchOrId($from);
      $to   = $locationTable->findFromBranchOrId($to);

      $distance = $this->getDistanceAndDays($from, $to);

      if (!$distance) {
        return $this->renderText(json_encode(array(
          'success' => false,
        )));
      }

      $nb_days = $distance[1];
      $i18n = sfContext::getInstance()->getI18N();

      $link = sprintf('<a class="reveal-ajax" data-reveal-refresh="0" href="%s">'.$i18n->__('min recommended no. of days').'</a>',
        $this->getController()->genUrl('listing/popupDistanceChart?from='.$from->getId().'&to='.$to->getId())
      );
      $message = $i18n->__('More than');
      $status = 'success label';

      if ($nb_days == $freeDays) {
        $message = $i18n->__('Equal to');
        $status  = 'label';
      }
      elseif ($nb_days > $freeDays) {
        $message = $i18n->__('Less than');
        $status = 'alert label';
      }

      $message .= ' '.$link.' ('.$nb_days.')';

      return $this->renderText(json_encode(array(
        'success'  => true,
        'from' => $from->getNameWithCountry('name'),
        'to' => $to->getNameWithCountry('name'),
        'distance' => $distance[0],
        'message'  => $message,
        'status'   => $status
      )));
    }

    public function executeGetRank(sfWebRequest $request)
    {
      $listing  = new Listing();
      $listingData     = $request->getParameter('listing');

      $locationTable = Doctrine::getTable('Locations');

      $from = $locationTable->findFromBranchOrId($listingData['pickup_location_id']);
      $listingData['pickup_location_id'] = $from ? $from->getId() : null;
      $to   = $locationTable->findFromBranchOrId($listingData['dropoff_location_id']);
      $listingData['dropoff_location_id'] = $to ? $to->getId() : null;
      $listingData['owner_id'] = $this->getUser()->getOwnerId();

      $listing->fromArray($listingData);

      $searchScore = new Transfercar\Ranking\SearchScore();
      $listing->setScore($searchScore->getScore($listing));
      $listing->setScoreDebug($searchScore->debug);

      $listingRank = new Transfercar\Ranking\ListingRank($locationTable->getConnection()->getDbh(), $this->getUser()->getCountry());
      $ranking = $listingRank->getRank($listing);

      return $this->renderText(json_encode(array(
        'ranking' => $ranking,
        'data'    => $listingData,
        'errors'  => array_map([$this->context->getI18N(), '__'], $searchScore->getErrors())
      )));
    }

    public function executeGetFreeDays(sfWebRequest $request)
    {
      $from     = $request->getParameter('from');
      $to       = $request->getParameter('to');

      $locationTable = Doctrine::getTable('Locations');

      $from = $locationTable->findFromBranchOrId($from);
      $to   = $locationTable->findFromBranchOrId($to);

      $distance = $this->getDistanceAndDays($from, $to);

      if ($distance === false)
      {
        return $this->renderText(json_encode(array(
          'success' => false,
        )));
      }

      return $this->renderText(json_encode(array(
        'success' => true,
        'from' => $from->getNameWithCountry('name'),
        'to' => $to->getNameWithCountry('name'),
        'nb_days' => $distance[1],
        'distance' => $distance[0]
      )));
    }

    private function getDistanceAndDays($from, $to)
    {
      $api      = new GoogleMapsDistancesMatrix();
      $distance = $api->getDistances(
        $from->getCountry()->get('domain') == 'us' ? $from->getNameWithCategory('name') : $from->getNameWithCountry('name'),
        $to->getCountry()->get('domain') == 'us' ? $to->getNameWithCategory('name') : $to->getNameWithCountry('name'),
        $this->getUser()->getUserDomain() == 'us' ? 'imperial' : 'metric'
      );

      if (false === $distance)
      {
        return false;
      }

      $nb_days = ceil($distance/Settings::getDistanceByDay($this->getUser()->getUserDomain()));

      return array($distance, $nb_days);
    }

    public function executeCreditPriceList()
    {
      $this->credits = VirtualCurrency::retreiveCreditsByCountryId($this
          ->getUser()
          ->getAttribute('profile')
          ->getCountryDomainId()
      );
    }

    public function executePopupDistanceChart(sfWebRequest $request)
    {
      $from     = $request->getParameter('from');
      $to       = $request->getParameter('to');

      $locationTable = Doctrine::getTable('Locations');

      $this->from = $locationTable->findFromBranchOrId($from);
      $this->to   = $locationTable->findFromBranchOrId($to);

      $distance = $this->getDistanceAndDays($this->from, $this->to);

      if (false !== $distance)
      {
        $this->distance = $distance[0];
        $this->nb_days  = $distance[1];
      }
    }

    public function executeExport()
    {
      $this->filters = $this->getUser()->getAttributeHolder()->getAll('sf_admin/listing/filters');
      $query = Doctrine::getTable('Listing')->createQuery();
      $this->addFiltersCriteria($query);

      $listings =  $query->execute ();

      $csv = fopen('php://temp/maxmemory:'. (5*1024*1024), 'r+');
      fputcsv($csv, [
        'Account','TCID','Pick Up','Pick Up Date','Pick Up Time','Dropoff','Dropoff Date','Dropoff Time',
        'Vehicle','First Name', 'Last Name','Phone','Email','Free days','Paid days','Fuel included'
      ]);
      foreach ($listings as $listing) {
        $request = $listing->getAcceptedRequest();
        $driver = $request->get('Driver');
        fputcsv($csv, [
          $listing->get('Owner')->get('company'),
          $request->getIncrementedId(),
          $listing->getPickup()->get('internal_name'),
          $request->get('pickup_date'),
          date('h:i', strtotime('+'.$request->get('pickup_time').' seconds', strtotime('today'))),
          $listing->getDropoff()->get('internal_name'),
          $request->get('drop_off_date'),
          date('h:i', strtotime('+'.$request->get('dropoff_time').' seconds', strtotime('today'))),
          $listing->getVehicle()->get('internal_name'),
          $driver->get('firstname'),
          $driver->get('lastname'),
          $driver->get('mobile_phone'),
          $driver->get('email'),
          $request->get('nb_free_days'),
          $request->get('nb_paid_days'),
          $listing->get('petrol_included') ? 'Y':'N'
        ]);
      }
      rewind($csv);
      $output = stream_get_contents($csv);

      $this->getResponse()->setHttpHeader('Content-Type', 'application/csv');
      $this->getResponse()->setHttpHeader('Content-Disposition', 'inline; filename=transfercar-export-' . date ( "Y-m-d" ) . '.csv;');
      $this->getResponse()->setHttpHeader('Cache-Control', 'private');

      return $this->renderText($output);
    }

    protected function addFiltersCriteria ($q)
    {
        $q
            ->innerJoin('Listing.PickupLocation pl')
            ->innerJoin('Listing.DropoffLocation dl')
            ->innerJoin('Listing.VehicleType v')
            ->leftJoin('Listing.ListedBy lb')
            ->select('Listing.*, pl.*, dl.*, lb.*, v.*, 1 as Manager')
            ->addWhere('Listing.owner_id = ?', $this->getUser()->getOwnerId())
            ->addWhere('Listing.PickupLocation.Category.country_id = ?', $this->getUser()->getCountry())
            ->addWhere('(Listing.status != ?)', Listing::STATUS_STANDBY)
        ;

        $ignoreStatusFilter = false;
        if (isset($this->filters['status']))
        {
          switch ($this->filters['status'])
          {
            case 'all':
              $ignoreStatusFilter = 'all';
              unset($this->filters['status']);
              break;
            case 'requested':
              $ignoreStatusFilter = 'requested';
              unset($this->filters['status']);
              $q
                ->addWhere('(Listing.status = ?)', Listing::STATUS_NEW)
                ->innerJoin('Listing.Requests lr WITH lr.status = ?', Request::STATUS_PENDING)
              ;
              break;
            case 'no_show':
              $ignoreStatusFilter = 'refunding';
              unset($this->filters['status']);
              $q
                ->addWhere('(Listing.status = ? or Listing.status = ? or Listing.status = ?)', array(
                  Listing::STATUS_ACCEPTED,
                  Listing::STATUS_PICKED_UP,
                  Listing::STATUS_DROPPED_OFF
                ))
                ->innerJoin('Listing.Requests lr WITH lr.no_show_fee = 1')
              ;
              break;
            case 'driver_cancelled':
              $ignoreStatusFilter = 'driver_cancelled';
              unset($this->filters['status']);
              $q
                ->innerJoin('Listing.Requests lr WITH lr.status = ?', Request::STATUS_CANCELED_BY_DRIVER)
                ->addWhere('lr.when_accepted is not null')
              ;
              break;
            case 'ro_cancelled':
              $ignoreStatusFilter = 'ro_cancelled';
              unset($this->filters['status']);
              $q
                ->innerJoin('Listing.Requests lr WITH lr.status = ?', Request::STATUS_OWNER_CANCELED)
              ;
              break;
            case 'disrupted':
              $ignoreStatusFilter = 'disrupted';
              unset($this->filters['status']);
              $q
                ->innerJoin('Listing.Requests lr WITH lr.is_withdrawn = 1')
              ;
              break;
            case 'rejected':
              $ignoreStatusFilter = 'disrupted';
              unset($this->filters['status']);
              $q
                ->innerJoin('Listing.Requests lr WITH lr.status = ?', Request::STATUS_REJECTED)
              ;
              break;
            case '':
              $q->addWhere('(Listing.status != ? and Listing.status != ? and Listing.status != ?)', array(Listing::STATUS_DELETED, Listing::STATUS_CLOSED, Listing::STATUS_STANDBY));
              $ignoreStatusFilter = '';
              break;
            default:
              $q->addWhere('Listing.status = ?', $this->filters['status']);
              $ignoreStatusFilter = $this->filters['status'];
              unset($this->filters['status']);
          }
        }

        foreach (array('date_available_from' => 'pickup_date', 'date_return_by' => 'drop_off_date') as $column => $reqColumn)
        {
          if (isset($this->filters[$column]) && $this->filters[$column] != '')
          {
            $dates  = explode(',', $this->filters[$column]);
            $first   = $dates[0];
            $last    = $dates[count($dates) - 1];
            if (strtotime($first) > strtotime($last))
            {
              // swap variables
              list($first, $last) = array($last, $first);
            }

            $q->addWhere('('.
              '(Listing.accepted_request_id is null and Listing.'.$column.' between ? and ?) '.
              ' OR '.
              '(Listing.accepted_request_id is not null and Listing.AcceptedRequest.'.$reqColumn.' between ? and ?)'.
              ')', array($first, $last, $first, $last));
          }
        }

        if (isset($this->filters['updated_at']) && $this->filters['updated_at'] != '')
        {
          $q->addWhere('Listing.updated_at between ? and ?', explode(',', $this->filters['updated_at']));
        }

        if (isset($this->filters['internal_reference']) && $this->filters['internal_reference'] !== '')
        {
          $q->leftJoin('Listing.AcceptedRequest ar');
          $q->addWhere('('.
              '(Listing.internal_reference LIKE ?) '.
              ' OR '.
              '(Listing.accepted_request_id is not null and concat(ar.Driver.firstname, \' \', ar.Driver.lastname) LIKE ?)'.
              ')', array('%'.$this->filters['internal_reference'].'%', '%'.$this->filters['internal_reference'].'%'));
        }

        if (isset($this->filters['id']) && $this->filters['id'] != '')
        {
          $id = explode('-', $this->filters['id']);
          $q->addWhere('Listing.id = ?', $id[0]);
        }

        if (isset($this->filters['ListedBy']) && $this->filters['ListedBy'] != '')
        {
          $q->addWhere('Listing.listed_by = ?', $this->filters['ListedBy']);
        }

        if (isset($this->filters['VehicleType']) && $this->filters['VehicleType'] !== '')
        {
          list($type, $id) = explode('_', $this->filters['VehicleType']);
          if ('owner' === $type)
          {
            $q->addWhere("Listing.owner_vehicle_type_id = ?", $id);
          }
          else
          {
            $q->addWhere("Listing.vehicle_type_id = ?", $id);
          }
        }

        foreach (array('Pickup', 'Dropoff') as $location)
        {
          if (isset($this->filters[$location.'Location']) && $this->filters[$location.'Location'] !== '')
          {
            list($type, $id) = explode('_', $this->filters[$location.'Location']);
            if ('owner' === $type)
            {
              $q->addWhere("Listing.owner_".strtolower($location)."_branch_id = ?", $id);
            }
            else
            {
              $q->addWhere("Listing.".strtolower($location)."_location_id = ?", $id);
            }
          }
        }

        parent::addFiltersCriteria($q);

        if ($ignoreStatusFilter === false && !isset($this->filters['status'])) {
          $q->addWhere('(Listing.status != ? and Listing.status != ? and Listing.status != ?)', array(Listing::STATUS_DELETED, Listing::STATUS_CLOSED, Listing::STATUS_STANDBY));
        }
        elseif (false !== $ignoreStatusFilter) {
          $this->filters['status'] = $ignoreStatusFilter;
        }
    }

    protected function addSortCriteria ($q)
    {
      parent::addSortCriteria($q);

      if ($sort_column = $this->getUser()->getAttribute('sort', null, 'sf_admin/listing/sort'))
      {
        $type = $this->getUser()->getAttribute('type', null, 'sf_admin/listing/sort') == 'asc' ? '' : 'desc';

        switch ($sort_column) {
          case 'Manager':
            $q->orderBy('Listing.ListedBy.firstname '.$type.', Listing.ListedBy.lastname '.$type);
            break;
          case 'view_id':
            $q->orderBy('Listing.id '.$type);
            break;
          case 'VehicleType':
            $q->orderBy('Listing.VehicleType.description '.$type);
            break;
          case 'PickupLocation':
            $q->orderBy('Listing.PickupLocation.name '.$type);
            break;
          case 'DropoffLocation':
            $q->orderBy('Listing.DropoffLocation.name '.$type);
            break;
          case 'date_available_from':
            $q->addSelect('IF(Listing.accepted_request_id, Listing.AcceptedRequest.PickupDate, Listing.DateAvailableFrom) real_pickup_date');
            $q->orderBy('real_pickup_date '.$type);
            break;
        }
      }
    }

    protected function getListingOrCreate ($id = 'id')
    {
        $listing = parent::getListingOrCreate($id);

        if ($listing->exists()) {
            $this->forward404Unless($listing->get('owner_id') === $this->getUser()->getOwnerId());
        }

        return $listing;
    }

    protected function deleteListing($listing)
    {
      $listing->remove(true);
    }
}
