<?php
use Transfercar\Stripe\TcStripeForm;

class mobileActions extends sfActions
{
    public function executeIndex()
    {
        $this->listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->where('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->execute();
    }

    public function executeAccept($request)
    {
        $ids = (array)$request->getParameter('request_ids', ( 
          array_fill_keys($this->getUser()->getAttribute('request_ids', ['-1']), 1) +
          array_fill_keys($this->getUser()->getAttribute('rejected_ids', ['-1']), -1)
        ));

        $acceptedIds = array_keys(array_filter($ids, function($value){ return (int)$value===1;}));
        $rejectedIds = array_keys(array_filter($ids, function($value){ return (int)$value===-1;}));

        $this->listings = count($ids) ? Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->where('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->whereIn('r.id', array_merge($acceptedIds, $rejectedIds, ['-1']))
            ->execute(): [];

        $rejectedRequests = count($rejectedIds) ? Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->where('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->whereIn('r.id', $rejectedIds)
            ->execute() : [];

        $this->price = $this->getPrice($this->listings, $rejectedIds);
        $this->form = $this->getAcceptForm($this->listings, $this->price);
        $this->rejectedRequests = $rejectedRequests;
        $this->rejected_ids = $rejectedIds;
    }

    public function executePay($request)
    {
        $ids = (array)$request->getParameter('request_ids', ( 
          array_fill_keys($this->getUser()->getAttribute('request_ids', ['-1']), 1) +
          array_fill_keys($this->getUser()->getAttribute('rejected_ids', ['-1']), -1)
        ));

        $acceptedIds = array_keys(array_filter($ids, function($value){ return (int)$value===1;}));
        $rejectedIds = array_keys(array_filter($ids, function($value){ return (int)$value===-1;}));

        $this->forward404Unless(count($ids));

        $this->listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->where('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->whereIn('r.id', count($acceptedIds) ? $acceptedIds : [-1])
            ->execute();

        $this->forward404Unless($request->isMethod('POST'));

        $this->price = $this->getPrice($this->listings);
        $this->form = $this->getAcceptForm($this->listings, $this->price);

        $this->form->bind($request->getParameter('confirm'));

        if ($this->form->isValid() && ($requests = $this->process($acceptedIds, $rejectedIds)) !== false) {
            $this->getUser()->setAttribute('request_ids', null);
            $this->getUser()->setAttribute('rejected_ids', null);
            $this->redirect('mobile/confirmed?request_ids='.implode(',', array_merge($acceptedIds, $rejectedIds)));
        }

        if (isset($this->form['credits'])) {
            $this->getUser()->setAttribute('request_ids', $acceptedIds);
            $this->getUser()->setAttribute('rejected_ids', $rejectedIds);
            return $this->redirect('mobile/selectCredits');
        }

        $this->setTemplate('accept');
        return $this->executeAccept($request);
    }

    public function executeConfirmed($request)
    {
        $ids = (array) $request->getParameter('request_ids', ["-1"]);

        if (strpos($ids[0], ',')!==false){
            $ids = explode(',', $ids[0]);
        }

        // $this->forward404Unless($ids[0] != '-1');
        $this->forward404Unless(count($ids));

        $this->listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->whereIn('r.status', [Request::STATUS_ACCEPTED, Request::STATUS_REJECTED])
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->whereIn('r.id', $ids)
            ->execute();
    }

    public function executeBookings($request)
    {
        $query = $this->applyBookingsFilters(Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.AcceptedListing l')
            ->innerJoin('r.Driver d')
            ->innerJoin('l.PickupLocation pl')
            ->innerJoin('l.DropoffLocation dl')
            ->innerJoin('l.OwnerPickupBranch opl')
            ->innerJoin('l.OwnerDropoffBranch ddl')
            ->where('l.owner_id = ?', $this->getUser()->getOwnerId()));

        $pager = new sfDoctrinePager('Request', 20);
        $pager->setPage($request->getParameter('page', 1));
        $pager->setQuery($query);
        $pager->init();

        $this->bookings = $pager->getResults();
        $this->pager = $pager;
    }

    public function executeListings($request)
    {
        $query = $this->applyListingsFilters(Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.PickupLocation pl')
            ->innerJoin('l.DropoffLocation dl')
            ->innerJoin('l.OwnerPickupBranch opb')
            ->innerJoin('l.OwnerDropoffBranch odb')
            ->where('l.owner_id = ?', $this->getUser()->getOwnerId()))
            ->addWhere('l.status != ?', Listing::STATUS_STANDBY);

        $pager = new sfDoctrinePager('Listing', 20);
        $pager->setPage($request->getParameter('page', 1));
        $pager->setQuery($query);
        $pager->init();

        $this->listings = $pager->getResults();
        $this->pager = $pager;
    }


    public function executeAddListing($request)
    {
      $oldPattern = sfConfig::get('app_date_time_pattern_'.$this->getUser()->getUserDomain());
      sfConfig::set('app_date_time_pattern_'.$this->getUser()->getUserDomain(), 'yyyy-mm-dd');
      $this->form = new OwnerListingForm(array(), array(
        'country' => $this->getUser()->getCountry(),
        'domain'  => $this->getUser()->getUserDomain(),
        'user'    => $this->getUser()->getAttribute('profile')
      ));
      $this->listing = new Listing();

      if ($request->isMethod('POST')) {
        $this->form->bind($request->getParameter('listing'));

        if ($this->form->isValid()) {
          $this->form->save($this->listing);
          $this->redirect('mobile/listings');
        }
      }

      //sfConfig::set('app_date_time_pattern_'.$this->getUser()->getUserDomain(), $oldPattern);
    }

    private function applyBookingsFilters($query)
    {
        $request = $this->getRequest();

        $status = $request->getParameter('status', '');
        switch ($status)
        {
          case '':
            $query->addWhere('(l.status != ? and l.status != ? and l.status != ?)', array(Listing::STATUS_DELETED, Listing::STATUS_CLOSED, Listing::STATUS_STANDBY));
            $ignoreStatusFilter = '';
            break;
          default:
            $query->addWhere('l.status = ?', $status);
        }
        


        if ($search = $request->getParameter('search')) {
            $query->addWhere('(l.id = ? or d.email like ? or concat(d.firstname, CONCAT(\' \', d.lastname)) like ?)', [$search, $search.'%', $search.'%']);
        }

        if ($tcid = $request->getParameter('tcid')) {
            $query->addWhere('l.id = ?', $tcid);
        }

        if ($driver_email = $request->getParameter('driver_email')) {
            $query->addWhere('d.email = ?', $driver_email);
        }

        $sort = $request->getParameter('sort');
        $dir = $request->getParameter('sort_direction', 'desc') == 'desc' ? 'desc': 'asc';

        switch($sort) {
            case 'pickup_date':
                $query->orderBy('r.pickup_date '.$dir);
                break;
            case 'dropoff_date':
                $query->orderBy('r.drop_off_date '.$dir);
                break;
            case 'pickup':
                $query
                    ->orderBy('pl.name '.$dir);
                break;
            case 'dropoff':
                $query
                    ->orderBy('dl.name '.$dir);
                break;
            default:
            case 'booking_date':
                $query->orderBy('r.when_accepted '.$dir);
                break;            
        }

        return $query;
    }

    private function getListingStatuses($status){
        switch ($status) {
            case 'listed':
                return [0];
            case 'booked':
                return [1,7,8,10,11];
            case 'cancelled':
                return [6];
            default:
                return [];
        }
    }

    private function applyListingsFilters($query)
    {
        $request = $this->getRequest();

        $status = $request->getParameter('status', 'all');

        switch ($status)
        {
          case 'all':
            $ignoreStatusFilter = 'all';
            break;
          case 'requested':
            $ignoreStatusFilter = 'requested';
            $query
              ->addWhere('(l.status = ?)', Listing::STATUS_NEW)
              ->innerJoin('l.Requests lr WITH lr.status = ?', Request::STATUS_PENDING)
            ;
            break;
          case 'refunding':
            $ignoreStatusFilter = 'refunding';
            $query
              ->addWhere('(l.status = ? or l.status = ? or l.status = ?)', array(
                Listing::STATUS_ACCEPTED,
                Listing::STATUS_PICKED_UP,
                Listing::STATUS_DROPPED_OFF
              ))
              ->innerJoin('l.Requests lr WITH lr.no_show_fee = 1')
            ;
            break;
          case '':
            $query->addWhere('(l.status != ? and l.status != ? and l.status != ?)', array(Listing::STATUS_DELETED, Listing::STATUS_CLOSED, Listing::STATUS_STANDBY));
            $ignoreStatusFilter = '';
            break;
          default:
            $query->addWhere('l.status = ?', $status);
            $ignoreStatusFilter = $this->filters['status'];
            unset($this->filters['status']);
        }

        if ($search = $request->getParameter('search')) {
            $query->addWhere('(l.id = ? or pl.name like ? or dl.name like ? or opb.internal_name like ? or opb.name like ? or odb.internal_name like ? or odb.name like ?)', [$search, $search.'%', $search.'%','%'.$search.'%', '%'.$search.'%','%'.$search.'%', '%'.$search.'%']);
        }

        if ($tcid = $request->getParameter('tcid')) {
            $query->addWhere('l.id = ?', $tcid);
        }

        if ($driver_email = $request->getParameter('driver_email')) {
            $query->innerJoin('r.Driver d')
                  ->addWhere('d.email = ?', $driver_email);
        }

        $sort = $request->getParameter('sort');
        $dir = $request->getParameter('sort_direction', 'desc') == 'desc' ? 'desc': 'asc';


        switch($sort) {
            default:
            case 'pickup_date':
                $query->orderBy('l.date_available_from '.$dir);
                break;
            case 'dropoff_date':
                $query->orderBy('l.date_return_by '.$dir);
                break;
            case 'pickup':
                $query
                    ->orderBy('pl.name '.$dir);
                break;
            case 'dropoff':
                $query
                    ->orderBy('dl.name '.$dir);
                break;
            
        }

        return $query;

        return $query;
    }

    private function process($acceptedIds, $rejectedIds)
    {
        $acceptedRequests = Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->whereIn('r.id', count($acceptedIds) ? $acceptedIds: [-1])
            ->addWhere('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->execute();

        $listingIds = [];

        $errors = $this->validateRequests($requests);
        
        if (empty($errors)) {
            foreach ($acceptedRequests as  $request) {
                $request->set('booking_type', 'mobile');
                if (!$request->accept($this->getUser()->getProfile()->getUsername())) {
                    $errors[] = ['type' => 'accept', 'request' => $request];
                }
                $listingIds[] = $request->get('listing_id');
            }

            // overlapping requests might have been rejected already when accepting others above
            // This will only reject non-overlapping requests (we differenciate rejected/surplus requests in stats)
            $rejectedRequests = Doctrine::getTable('Request')
              ->createQuery('r')
              ->innerJoin('r.Listing l')
              ->whereIn('r.id', count($rejectedIds) ? $rejectedIds : [-1])
              ->addWhere('r.status = ?', Request::STATUS_PENDING)
              ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
              ->execute();

            foreach ($rejectedRequests as $request) {
              if (in_array($request->get('listing_id'), $listingIds)) {
                $request->status = Request::STATUS_OTHER_CHOOSE;
                $request->save();
                // other choose
                sfContext::getInstance()->getEventDispatcher()->notify(new sfEvent($request, 'request.rejected'));
              } else {
                $request->reject();
              }
            }
        }

        if (!empty($errors)) {
            $this->errors = $errors;
            return false;
        }

        return true;
    }

    private function validateRequests($requests)
    {
        $errors = [];

        foreach ($requests as $request) {
            if (!$request->isStatus(Request::STATUS_PENDING)) {
                $errors[] = ['type' => 'checksum', 'request' =>  $request];
            }
        }

        return $errors;
    }

    private function getPrice($listings, $rejectedIds = [])
    {
        $total = 0;
        foreach($listings as $listing) 
        {
            foreach($listing->get('Requests') as $request) {
                if (!in_array($request->getId(), $rejectedIds)) {
                    $total += $listing->getTotalPrice($request);
                }
            }
        }
        return $total;
    }

    private function getAcceptForm($listings, $price)
    {

        return new RequestAcceptForm(array(), array(
            'user' => $this->getUser()->getAttribute('profile'),
            'price' => $price,
            'country' => $this->getUser()->getAttribute('profile')->getCountryDomainId(),
            'renderer' => function ($widget, $inputs) {
                $rows = array();
                foreach ($inputs as $input)
                {
                  $rows[] = $widget->renderContentTag('li', $input['input'].' '.preg_replace('#(\(.*\))</label>#', '</label><div class="help">$1</div>', $input['label']), array('class' => 'list-group-item'));
                }

                return $widget->renderContentTag('ul', implode($widget->getOption('separator'), $rows), array('class' => 'list-group'));
            }
        ));
    }


    /* payment actions */

    /**
     * Init session payment info parameter
     *
     */
    protected function initPayment()
    {
        $this->payment_info = $this->getUser()->getAttribute("payment_info", "");
        $this->nb = 0;
        $this->credits = 0;
        $this->revenue = 0;
        $this->ids = $this->getUser()->getAttribute('request_ids', ['-1']);

        $this->listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->where('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->whereIn('r.id', $this->ids)
            ->execute();

        $this->price = $this->getPrice($this->listings);
        foreach ($this->listings as $listing) {
            foreach($listing->get('Requests') as $request) {
                $this->credits += $listing->getTotalPrice($request);
                $this->revenue += $listing->getRevenue($request);
                $this->nb++;
            }
        }
    }

    /**
     * Check valid credit(virtual currency)
     *
     */
    protected function checkValidCredit($credit, $credits)
    {
        foreach ($credits as $item){
            if ($item["id"] == $credit) return true;
        }

        return false;
    }

    public function executeSelectCredits($request)
    {
        $this->ids = (array) $request->getParameter('request_ids', $this->getUser()->getAttribute('request_ids', ['-1']));
        $this->rejected_ids = (array) $request->getParameter('rejected_ids', $this->getUser()->getAttribute('rejected_ids', ['-1']));

        $this->forward404Unless(count($this->ids) || count($this->rejected_ids));

        $this->listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->where('r.status = ?', Request::STATUS_PENDING)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->whereIn('r.id', $this->ids)
            ->execute();

        $this->price = $this->getPrice($this->listings);
        $this->form = $this->getAcceptForm($this->listings, $this->price);
    }

    public function executeSelectCreditCard($request)
    {
        $values = $request->getParameter('confirm');
        $this->getUser()->setAttribute("payment_info", $values['credits']);
        $this->redirect('mobile/chargeAccount');
    }

    public function executePayWithCard($request)
    {
        $this->initPayment();
        $this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId());

        if (empty($this->payment_info) || !$this->checkValidCredit($this->payment_info, $this->credits)){
            $this->redirect("mobile/selectCredits");
        }
        $this->credit_info = VirtualCurrency::retreiveCreditInfo($this->payment_info);

        // If we have request for pay with stored
        // card, redirect to the proper action
        if($request->getParameter('balance_card', 'new') == 'existing'){
            $this->forward('mobile', 'payWithStoredCard');
        }
        $user = $this->getUser()->getAttribute('profile');

        $this->form = new TcStripeForm(
            ['amount' => $this->credit_info["price"]], 
            [
                'user_id'  => $user->getOwnerId(),
                'credits'  => $this->credit_info["credit"],
                'currency' => strtolower(sfConfig::get('app_currency_'.$user->get('Country')->get('domain'))),
                'country_id' => $this->getUser()->getAttribute('profile')->getCountryDomainId(),
                'email' => $this->getUser()->getAttribute('profile')->get('email')
            ]
        );
        $this->amount = $this->credit_info["price"];

        if (!$request->hasParameter('payment')) {
            $this->getResponse()->setStatusCode(400);
            return sfView::SUCCESS;
        }

        $this->form->bind($request->getParameter('payment'));

        if(!$this->form->isValid()){
            $this->getResponse()->setStatusCode(400);
            return sfView::SUCCESS;
        }

        $particular = sprintf('Transfercar credits', $this->credit_info["price"]);
        $description = substr('CRD/'.$user->getCompany(), 0, 50);
        if (!$cct = $this->form->processPayment($description, $particular, $this->form->getValues())) {
            $this->getResponse()->setStatusCode(400);
            //redirect to page with error info
            return sfView::SUCCESS;
        }

        return $this->redirect('mobile/checkTransactionStatus');
    }

    /**
     * Pay with stored card
     *
     * @param sfWebRequest $request HTTP request object
     */
    public function executePayWithStoredCard($request)
    {
        $this->initPayment();
        $this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId());

        if (empty($this->payment_info) || !$this->checkValidCredit($this->payment_info, $this->credits)){
            $this->redirect("mobile/selectCredits");
        }

        $this->credit_info = VirtualCurrency::retreiveCreditInfo($this->payment_info);
        $user = $this->getUser()->getAttribute('profile');
        // If we have request for pay with new
        // card, redirect to the proper action
        if($request->getParameter('balance_card', 'new') != 'existing'){
            $this->forward('mobile', 'payWithCard');
        }

        if(!$request->hasParameter('card_id')){
            // Should we put default value for amount
            $this->amount = intval($this->credit_info["price"]);
            if($this->amount == 0){
                $this->amount = '';
            }

            $this->cards = CreditCard::user_cards_list();
            krsort($this->cards);
            return sfView::SUCCESS;
        }

        $card_id = intval($request->getParameter('card_id'));

        // Check if this card belongs to current user
        $card = Doctrine_Query::create()
            ->from('CreditCard')
            ->where('id = ?', $card_id)
            ->addWhere('owner_id = ?', $user->owner_id)
            ->addWhere('holder_name LIKE ?', 'src_%')
            ->fetchOne();

        if(!is_object($card)){
            $this->renderText('Invalid card id. Please contact with system administrator');
            return sfView::NONE;
        }

        // OK, form is valid, prepare CC transaction and
        // create payment record. By default
        // add requested amount to user balance
        // It will be decreased by cron script
        // if card transactions will fail
        $amount = intval($request->getParameter('amount'));
        $receipt = intval($request->getParameter('receipt', 0));

        $user->refresh(); // we are updating 'account balance' field, which
                          // can be modified by cron script, therefore we
                          // have to have sure, that we have fresh data
        $this->form = new TcStripeForm(
            ['amount' => $this->credit_info["price"]], 
            [
                'is_new_token' => false,
                'user_id' => $user->getOwnerId(),
                'credits' => $this->credit_info["credit"],
                'currency' => strtolower($user->get('Country')->get('domain').'d'),
                'country_id' => $this->getUser()->getAttribute('profile')->getCountryDomainId(),
                'email' => $this->getUser()->getAttribute('profile')->get('email')
            ], false
        );

        $particular = sprintf('Transfercar credits', $this->credit_info["price"]);
        $description = substr('CRD/'.$user->getCompany(), 0, 50);
        $values = [
            'amount' => $this->credit_info["price"],
            'stripeToken' => $card->get('holder_name')
        ];
        $this->form->bind($values);
        if (!$this->form->isValid() || (!$cct = $this->form->processPayment($description, $particular, $values, $card))) {
            $this->amount = intval($this->credit_info["price"]);
            if($this->amount == 0){
                $this->amount = '';
            }
            $this->cards = CreditCard::user_cards_list();
            return sfView::SUCCESS;
        }

        return $this->redirect('mobile/checkTransactionStatus');
    }

    public function executeCheckTransactionStatus($request)
    {
        $this->initPayment();
        $this->ids = (array) $request->getParameter('request_ids', $this->getUser()->getAttribute('request_ids', ['-1']));
        $this->rejected_ids = (array) $request->getParameter('rejected_ids', $this->getUser()->getAttribute('rejected_ids', ['-1']));

        $this->forward404Unless(count($this->ids));

        $this->form = $this->getAcceptForm($this->listings, $this->price);
    }


    /*
    * Charge account - get required information and redirect
    * to payWithCard action
    */
    public function executeChargeAccount()
    {
        $this->initPayment();
        $this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId());

        if (empty($this->payment_info) || !$this->checkValidCredit($this->payment_info, $this->credits)){
            $this->redirect("mobile/selectCredits");
        }

        $this->credit_info = VirtualCurrency::retreiveCreditInfo($this->payment_info);

        $uid = $this->getUser()->getAttribute('profile')->id;

        // Get user info with informations about its credit cards
        $this->user = Doctrine_Query::create()
            ->from('User u')
            ->leftJoin('u.CreditCards')
            ->where('u.id = ?', $uid)
            ->fetchOne();

        return sfView::SUCCESS;
    }
}