<?php echo decorate_with('mobile_layout') ?>
<?php slot('breadcrumb', 'Add listing') ?>

<div class="row mb-4">
  <div class="col-12">
    <div class="card">
      <div class="card-header pb-0 px-3">
        <h6 class="mb-0">
          Add a new listing
        </h6>
      </div>
      <div class="card-body pt-4 p-3 pb-4">
      <form action="<?php echo url_for('mobile/addListing') ?>" method="post" id="listing_form" name="listing_form">

      <h5>1. <?php echo __('Vehicles details') ?> <?php echo tooltip('listing_form_vehicle_details') ?></h5>
      <div class="row locations">
        <div class="col-12">
          <?php echo render_row($form['vehicle_type_id'], 'Vehicle to relocate', array('class' => 'form-control', 'data-placeholder' => __('Select vehicle or category')), 'is_invalid') ?>
        </div>
      </div>
      <div class="row">
        <div class="col-4">
          <?php echo render_row($form['nb_listings'], 'Quantity', array('class' => 'form-control'), 'is_invalid') ?>
        </div>

        <div class="col-8">
          <?php echo render_row($form['internal_reference'], 'Reference', array('class' => 'form-control'), 'is_invalid') ?>
        </div>
      </div>

      <hr/>
      <h5>2. <?php echo __('Journey details') ?></h5>
      <div class="row locations">
        <div class="col-12">
          <div class="panel">
            <?php echo render_row($form['pickup_location_id'], 'Pickup location', array('class' => 'form-control', 'data-placeholder' => __('Select branch or location'))) ?>
            <?php echo str_replace('text/javascript', 'text/template', render_row($form['date_available_from'], 'Earliest pickup date', array('class' => 'form-control datepicker'), 'is_invalid')) ?>
            <div class="row">
              <div class="col-6">
                <?php echo render_row($form['pickup_time'], 'Earliest pickup', array('class' => 'form-control'), 'is_invalid') ?>
              </div>
              <div class="col-6">
                <?php echo render_row($form['latest_pickup_time'], 'Latest pickup', array('class' => 'form-control'), 'is_invalid') ?>
              </div>
            </div>
          </div>
        </div>
        <hr/>
        <div class="col-12">
          <div class="panel">
            <?php echo render_row($form['dropoff_location_id'], 'Dropoff location', array('class' => 'form-control', 'data-placeholder' => __('Select branch or location')), 'is_invalid') ?>
            <?php echo str_replace('text/javascript', 'text/template', render_row($form['date_return_by'], 'Latest dropoff date', array('class' => 'form-control datepicker'))) ?>
            <div class="row">
              <div class="col-6">
                <?php echo render_row($form['dropoff_time'], 'Earliest dropoff', array('class' => 'form-control'), 'is_invalid') ?>
              </div>
              <div class="col-6">
                <?php echo render_row($form['latest_dropoff_time'], 'Latest dropoff', array('class' => 'form-control'), 'is_invalid') ?>
              </div>
            </div>
          </div>
        </div>
      </div>

      <div class="row mt-4">
        <div class="col-12 extras">
          <h5>3. <?php echo __('What\'s included') ?></h5>
            <div class="row">
              <div class="col-4">
                <?php echo $form['petrol_included']->renderLabel($form['petrol_included']->render(array('class' => '')).' '.__('Fuel')) ?>
              </div>
            <?php if ($form->getOption('domain') != 'za'): ?>

              <div class="col-4">
                <?php echo $form['ferry_included']->renderLabel($form['ferry_included']->render(array('class' => '')).' '.__('Ferry')) ?>
              </div>
            <?php endif ?>
              <div class="col-4">
                <?php echo $form['expenses_included']->renderLabel($form['expenses_included']->render(array('class' => '')).' '.__('Expenses')) ?>
              </div>
            </div>
            <div class="row">
              <div class="col-4 pe-0">
                <?php echo $form['insurance_included']->renderLabel($form['insurance_included']->render().' '.__('Insurance'), array('class' => '', 'readonly' => true)) ?>
                <?php echo $form['insurance_included']->renderError() ?>
              </div>
              <div class="col-8">
                <?php echo $form['insurance_policy']->render(['class' => 'form-control']) ?>
                <?php echo $form['insurance_policy']->renderError() ?>
              </div>
            </div>
        </div>
      </div>
      <hr/>
      <div class="row">
        <div class="col-6">
          <?php echo render_row($form['days_allowed'], 'Free days allowed', array('class' => 'form-control '.(!$listing->exists() && $sf_params->get('id') ? 'manually_changed' : '')), 'is_invalid') ?>
        </div>
        <div class="col-6">
          <?php echo render_row($form['kms_allowed'], __('Kms allowed'), ['class' => 'form-control']) ?>
        </div>
      </div>

      <div class="row">
        <div class="col-6">
          <?php echo render_row($form['paid_days_allowed'], 'Extra paid days', array('class' => 'form-control'), 'is_invalid') ?>
        </div>
        <div class="col-6">
          <?php echo render_row($form['price_per_day'], 'Price per day ($)', array('class' => 'form-control'), 'is_invalid') ?>
        </div>
      </div>

      <div class="row mt-4 mb-4">
        <div class="col-12">
          <h5>4. <?php echo __('Driver requirements') ?></h5>
          <?php echo render_row($form['min_age'], 'Minimum driver age', array('class' => 'form-control'), 'is_invalid') ?>
        </div>
      </div>

      <?php echo $form->renderCSRF(); ?>
      <?php echo $form['kms_estimated']->render() ?>
      </form>
    </div>
  </div>
</div>


<?php use_javascript('/assets/js/plugins/moment.min.js') ?>
<?php use_javascript('/assets/js/plugins/flatpickr.min.js') ?>
<?php append_slot('javascripts') ?>

<script type="text/javascript">
  if (document.querySelector(".datepicker")) {
    flatpickr(".datepicker", {
      minDate: "<?php echo date('Y-m-d') ?>",
      dateFormat: "Y-m-d",
      altFormat: "Y-m-d",
      altInput: true,
      parseDate: function(d) {
        return new Date(d);
      }
    }); // flatpickr
  }
</script>
<style type="text/css">
.pending-menu { display: block }
ul.error_list {
    list-style-type: none;
    padding: 0;
    color: red;
    width: 100%;
    margin-top: 0.25rem;
    font-size: .875em;
    color: #dc3545;
}

</style>
<div class="pending-menu">
  <div class="row">
    <div class="col-1">
      <a href="<?php echo url_for('mobile/listings') ?>" ><i class="fa fa-regular fa-arrow-left"></i></a>
    </div>
    <div class="col-6">
      <p class="text-sm mb-0 text-uppercase font-weight-bold" id="price">
        &nbsp;
      </p>
    </div>
    <div class="col-5">
        <button type="submit" form="listing_form" class="btn btn-primary">Add listing</button>
      </form>
    </div>
  </div>
</div>
<?php end_slot() ?>