<?php decorate_with('mobile_layout') ?>
<?php use_javascript("/assets/js/requests.js") ?>

<?php if (!count($listings)): ?>
  <div class="row mb-4">
    <div class="col-12">
      <div class="card">
        <div class="card-header pb-0 px-3">
          <h6 class="mb-0">
            You don't have any pending request yet!
          </h6>
        </div>
        <div class="card-body">
          You'll get notified by email as soon as a request is available.

          <?php if (sfConfig::get('app_tc_env') !== 'prod'): ?>
            <br/><br/><hr/>

            <p class="badge text-bg-info">
              <a href="#" id="generate-requests2"><i class="fa fa-solid fa-gear"></i> Testing: Generate requests</a>
            </p>
          <?php endif ?>
        </div>
      </div>
    </div>
  </div>
<?php endif ?>

<?php foreach ($listings as $listing): ?>
  <?php $total = $nb = 0; ?>
  <div class="row mb-4">
    <div class="col-12">
      <div class="card">
        <div class="card-header pb-0 px-3">
          <h6 class="mb-0">
            TCID <?php echo $listing->getId() ?> 
            <span class="float-end"><?php echo $listing->getVehicle() ?></span>
          </h6>
          <p class="text-sm mb-0 font-weight-bold"><?php echo $listing->getTitle() ?></p>
        </div>
        <div class="card-body pt-4 p-3 pb-1">
          <ul class="list-group">
            <?php foreach($listing->get('Requests') as $request): ?>
              <?php if ($request->isStatus(Request::STATUS_PENDING)): ?>
              <?php $overlaps = $request->hasOverlappingRequest() ?>
                <li id="req_<?php echo $request->getId() ?>" rel="<?php echo 'driver-'.$request->get('driver_id') ?>" class="list-group-item border-0 p-2 mb-2 bg-gray-100 border-radius-lg <?php echo 'driver-'.$request->get('driver_id') ?><?php echo $overlaps ? ' overlaps' : '' ?>">
                  <?php include_partial('mobile/request', ['request' => $request]) ?>
                </li>
              <?php endif ?>
            <?php endforeach ?>
          </ul>
        </div>
        <div>
          <div class="text-sm text-normal p-3 pt-0">
              Listings available  <span data-nb-available="<?php echo $listing->get('nb_listings') ?>" id="nb_<?php echo $listing->getId() ?>" class="nb"><?php echo $listing->get('nb_listings') ?></span> / <?php echo $listing->get('nb_listings') ?>
          </div>
        </div>
      </div>
    </div>
  </div>
  <hr style="border:1px solid;opacity:1;"/>
<?php endforeach ?>
<?php use_stylesheet('/assets/css/requests.css') ?>
<?php append_slot('javascripts') ?>
<div class="pending-menu">
  <div class="row">
    <div class="col-8">
      <p class="text-sm mb-0 text-uppercase font-weight-bold" id="nb_requests">&nbsp;</p>
      <p class="text-sm mb-0 text-uppercase font-weight-bold" id="total_price">&nbsp;</p>
    </div>
    <div class="col-4">
      <form name="request_form" id="request_form" action="<?php echo url_for('mobile/accept') ?>" method="post" autocomplete="off">
        <button type="submit" href="<?php echo url_for('mobile/accept') ?>" class="btn btn-primary">Select</button>
      </form>
    </div>
  </div>
</div>

<script type="text/javascript">
(function($) {
    function updateTotal() {
        var total = 0;
        var nb = $('[name*="request_ids"]:checked').each(function(){
            total += parseInt($(this).data('price'));
        }).length;
    }

    $('.btn-reject + label, .btn-accept + label').on('click', function(e){
      checkbox=document.querySelector('#'+$(this).attr('for'));
      if (checkbox.checked) {
        e.preventDefault();
        checkbox.checked = false;
        $(checkbox).trigger('change');
        $(this).blur();
      }
    });

    function initSwitches(){
      $('[name*="request_ids"]').on('change', function(){
          var nb_checked = $(this).parents('.card:first').find('[value=1]:checked').length,
              listing_id = $(this).data('listing-id'),
              nb_available = parseInt($('#nb_'+listing_id).data('nb-available')),
              is_overlaps = $(this).hasClass('overlaps');

          $('#nb_'+listing_id).text(nb_available-nb_checked);
          updateTotal();
          
          // disable others requests is max listing is reach
          $(this).parents('.card:first')
            // select all radio buttons that has not been disabled already because they overlap
              .find(':radio:not(:checked):not(.overlapping)[value=1]')
              .prop('disabled', nb_checked >= nb_available);
          // return;
          $('li:has(:radio)').removeClass('disabled');

          if (parseInt($(this).val()) == 1) {
            // rel = driver_id, looking for other requests with the same user ID
            var cssClass = '.'+$(this).parents('li:first').attr('rel')+':not(#'+$(this).parents('li:first').attr('id')+') [name*="request_ids"]';

            $(cssClass)
              .prop('disabled', $(this).is(':checked'))
              .toggleClass('overlapping', $(this).is(':checked'));

            $('li:has(:radio:disabled)').addClass('disabled');
          }
      });
    }
    initSwitches();
    updateTotal();
    
    $('li:has(:radio:disabled)').addClass('disabled');

    $('.reject-btn').on('click', function(){
      var link = this;
      $(this).html('<i class="fa-solid fa-spinner fa-spin"></i>');
      $.ajax({
        url: link.href,
        dataType: 'json',
        success: function() {
          var li = $(link).parents('li:first'),
              card = $(li).parents('.row.mb-4');

          li.remove();
          if (card.has('li').length == 0) {
            card.next().remove(); //removes hr
            card.remove();

            if ($('.card').length == 0) {
              document.location.href="<?php echo url_for('mobile/index') ?>";
            }
          }
        }
      });

      return false;
    });

    <?php if (sfConfig::get('app_tc_env') !== 'prod'): ?>
      $('#generate-requests,#generate-requests2').on('click', function(){
        $(this).find('.fa-gear').addClass('fa-spin');
        
        $.ajax({url: '<?php echo url_for('bookings/generate') ?>', success: function(){
          $.ajax({url: '<?php echo url_for('bookings/generate') ?>', success: function(){
            $.ajax({url: '<?php echo url_for('bookings/generate') ?>', success: function(){
              document.location.href=document.location.origin+document.location.pathname;
            }});
          }});
        }});      
      });
    <?php endif ?>
})(jQuery);
</script>
<?php end_slot() ?>