<?php decorate_with('mobile_layout') ?>
<?php slot('breadcrumb', 'Listings') ?>



<div class="row mb-4">
  <div class="col-12">
    <div class="card">
      <div class="card-header pb-0 px-3">
        <h6 class="mb-0">
          Your listings <button class="float-end btn" data-bs-toggle="collapse" href="#collapseFilter" role="button" aria-expanded="false" aria-controls="collapseFilter"><i class="fa fa-solid fa-filter"></i></button>
        </h6>
      </div>
      <div class="card-body pt-4 p-3 pb-1">
        <div class="collapse" id="collapseFilter">
          <form action="" method="post">
            <div class="form-group">
              <?php $statuses = ListingTable::getStatuses() ?>
              <?php echo select_tag('status', options_for_select(array_map('__', $statuses), $sf_params->get('status')), ['class' => 'form-control']) ?>
            </div>

            <div class="form-group">
              <input placeholder="TCID, pickup or dropoff" type="text" name="search" class="form-control" value="<?php echo $sf_request->getParameter('search') ?>"/>
            </div>

            <div class="row mb-3">
              <div class="col-7">
                <div class="input-group">
                  <div class="input-group-text"><i class="fa-solid fa-arrow-up-arrow-down"></i></div>
                  <select class="form-control" name="sort">
                    <option <?php echo ($sf_request->getParameter('sort', 'pickup_date') == 'pickup_date') ?' selected ':''?> value="pickup_date">Pickup date</option>
                    <option <?php echo ($sf_request->getParameter('sort') == 'dropoff_date') ?' selected ':''?> value="dropoff_date">Dropoff date</option>
                    <option <?php echo ($sf_request->getParameter('sort') == 'pickup') ?' selected ':''?> value="pickup">Pickup location</option>
                    <option <?php echo ($sf_request->getParameter('sort') == 'dropoff') ?' selected ':''?> value="dropoff">Dropoff location</option>
                  </select>
                </div>
              </div>
              <div class="col-5">

                <select class="form-control" name="sort_direction">
                  <option <?php echo ($sf_request->getParameter('sort_direction') == 'asc') ?' selected ':''?> value="asc">Ascending</option>
                  <option <?php echo ($sf_request->getParameter('sort_direction', 'desc') == 'desc') ?' selected ':''?> value="desc">Descending</option>
                </select>
              </div>
            </div>

            <button class="btn" type="submit">Filter</button>
          </form>
        </div>
      </div>
    </div>
  </div>
</div>


<?php if (!count($listings)): ?>
  <div class="row mb-4">
    <div class="col-12">
      <div class="card">
        <div class="card-header pb-0 px-3">
          <h6 class="mb-0">
            You don't have any listing  yet!
          </h6>
        </div>
        <div class="card-body">
          Create one <?php echo link_to('now', 'listing/create') ?>.
        </div>
      </div>
    </div>
  </div>
<?php endif ?>


<div class="accordion accordion-flush" id="accordion-listings">
  <?php foreach ($listings as $listing): ?>

  <div class="accordion-item">
    <h4 class="accordion-header" id="flush-headingOne">
      <button class="accordion-button collapsed" type="button" data-bs-toggle="collapse" data-bs-target="#flush-collapse<?php echo $listing->getId() ?>" aria-expanded="false" aria-controls="flush-collapse<?php echo $listing->getId() ?>">
        
        <div class="ps-2 w-100">
          <?php include_partial('mobile/booking_status', ['listing' => $listing]) ?>
          <div>
            <i class="fa fa-solid fa-route"></i>
            <?php echo $listing->get('OwnerPickupBranch')->get('internal_name') ?> to <?php echo $listing->get('OwnerDropoffBranch')->get('internal_name') ?>
          </div>
          <div>
            <i class="fa fa-solid fa-calendar"></i>
            <?php echo tc_format_date($listing->get('date_available_from'), 'short') ?>-
            <?php echo tc_format_date($listing->get('date_return_by'), 'short') ?>
          </div>
        </div>
      </button>

    </h4>
    <div id="flush-collapse<?php echo $listing->getId() ?>" class="accordion-collapse collapse" aria-labelledby="flush-headingOne" data-bs-parent="#accordion-listings">
      <div class="accordion-body">
         <div class="col-12">
            <ul class="list-group">
                <li class="list-group-item border-0 d-flex p-0 pt-2 mb-2 bg-gray-100 border-radius-lg">
                    <div class="d-flex flex-column">
                      <span class="mb-1 text-xs">TCID: <span class="text-dark ms-sm-2 font-weight-bold"><?php echo $listing->getId() ?></span></span>
                      <span class="mb-1 text-xs">Vehicle: <span class="text-dark ms-sm-2 font-weight-bold"><?php echo $listing->get('OwnerVehicleType')->get('name') ?></span></span>
                      <span class="mb-1 text-xs">Type: <span class="text-dark ms-sm-2 font-weight-bold"><?php echo $listing->get('OwnerVehicleType')->get('VehicleType')->get('description') ?></span></span>
                      <span class="mb-1 text-xs"><?php echo __('Relocation cost') ?>: <span class="text-dark ms-sm-2 font-weight-bold"><?php echo $listing->get('price') ?></span></span>
                    </div>
                </li>
            </ul>
        </div>
      </div>
    </div>
  </div>
  <?php endforeach ?>
</div>

<?php if ($pager->haveToPaginate()): ?>
  <?php $filters = http_build_query(array_filter([
    'search' => $sf_params->get('search'),
    'status' => $sf_params->get('status'),
    'sort'   => $sf_params->get('sort'),
    'sort_direction'  => $sf_params->get('sort_direction'),
  ])); ?>
  <div class="row mt-4 mb-4">
    <div class="col-12">
      <nav aria-label="Booking page navigation">
        <ul class="pagination justify-content-center">

          <li class="page-item">
            <a href="<?php echo url_for('mobile/listings?page='.$pager->getFirstPage().($filters ? '&'.$filters:'')) ?>" class="page-link" aria-label="First"><span aria-hidden="true">&laquo;</span></a>
          </li>
          <li class="page-item disabled">
            <a href="<?php echo url_for('mobile/listings?page='.$pager->getPreviousPage().($filters ? '&'.$filters:'')) ?>" class="page-link" aria-label="Previous"><span aria-hidden="true">&lt;</span></a>
          </li>
          <?php foreach($pager->getLinks(3) as $page): ?>
            <li class="page-item<?php echo ($sf_request->getParameter('page', 1) == $page) ? ' active' : '' ?>"><a href="<?php echo url_for('mobile/listings?page='.$page.($filters ? '&'.$filters:'')) ?>"class="page-link"><?php echo $page ?></a></li>
          <?php endforeach ?>
          <li class="page-item">
            <a href="<?php echo url_for('mobile/listings?page='.$pager->getNextPage().($filters ? '&'.$filters:'')) ?>" class="page-link" aria-label="Next"><span aria-hidden="true">&gt;</span></a>
          </li>

          <li class="page-item">
            <a href="<?php echo url_for('mobile/listings?page='.$pager->getLastPage().($filters ? '&'.$filters:'')) ?>" class="page-link" aria-label="First"><span aria-hidden="true">&raquo;</span></a>
          </li>
        </ul>
      </nav>
    </div>
  </div>
<?php endif ?>
<?php if (false): ?>
<a href="<?php echo url_for('mobile/addListing') ?>" id="add_button" class="btn btn-primary">
  <i class="fa fa-plus fa-2x"></i>
</a>
<?php endif ?>
<?php append_slot('javascripts') ?>
<style type="text/css">
  .accordion {
    --bs-accordion-bg:  #fff;
    --bs-accordion-color:  #000;
  }

  .accordion-item {
    border-bottom: 1px solid #ccc;
  }

  .accordion-item:first-of-type {
    border-top-left-radius: 1rem!important;
    border-top-right-radius: 1rem!important;
  }

  .accordion-body .list-group {
    margin-top: 10px!important;
  }

  .accordion-body .list-group-item {
    padding: 5px!important;
  }

  .page-item.active .page-link {
    color:  #fff;
  }

  .accordion-header .float-end.badge {
    margin-top: -5px;
    margin-right: -20px;
  }


  #add_button {
    border-radius: 50%;
    position: fixed;
    bottom: 50px;
    right: 10px;
    padding: 9px 11.5px;
  }
</style>
<?php end_slot() ?>