<?php

use Transfercar\Geoloc\GoogleMapsDistancesMatrix;

/**
 * rental actions.
 *
 * @package    symfony
 * @subpackage rental
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 5647 2007-10-23 17:25:20Z Eric.Fredj $
 */
class rentalsActions extends autorentalsActions
{
    public function preExecute ()
    {
      if (!in_array($this->getRequestParameter('action'), array('list', 'edit', 'create')))
      {
        $this->getResponse()->addStylesheet('/sf/sf_admin/css/main');
      }
    }

    public function executeView(sfWebRequest $request)
    {
      $this->rental = $this->getRentalOrCreate();
      $this->forward404Unless($this->rental->exists());
    }

    public function executeEdit(sfWebRequest $request = null)
    {
        $rental = $this->getRentalOrCreate();
        $rental->set('owner_id', $this->getUser()->getOwnerId());

        //$this->forward404Unless($rental->exists());

        $this->form = new OwnerRentalForm(array(), array(
          'country' => $this->getUser()->getCountry(),
          'domain'  => $this->getUser()->getUserDomain(),
          'user'    => $this->getUser()->getAttribute('profile')
        ));

        $this->form->setDefaultsFromRental($rental);
        $this->rental = $rental;

        if ($request->isMethod('POST'))
        {
          $this->form->bind($request->getParameter('rental'), $request->getFiles('rental'));

          if ($this->form->isValid())
          {
            if ($this->getUser()->hasCredential('IS_IMPERSONATED'))
            {
              $rental->set('admin_id', $this->getUser()->getAttribute('admin_id'));
            }
            else
            {
              $rental->set('admin_id', null);
            }

            $this->form->save($rental);
            if ($request->getParameter('id')) {
              $this->getUser()->setFlash('notice', 'Your rental has been saved');
            } else {
              $this->getUser()->setFlash('notice', 'Your rental has been added');
              $this->dispatcher->notify(new sfEvent($rental, 'ro.rental.created'));
            }

            return $this->redirect('rentals/index');
          }
        }
    }

    public function executeManage()
    {
      $this->rental = $this->getRentalOrCreate();
      $this->requests = $this->rental->get('Enquiries');
    }

    public function executeAccept($request)
    {
      $this->enquiry = Doctrine::getTable('Enquiry')->find($request->getParameter('id'));
      $this->form = new BaseForm();

      if ($request->isMethod('POST')) {
        
        $this->enquiry->accept();

        $this->redirectUnless($request->isXmlHttpRequest(), 'rentals/confirm?id='.$this->enquiry->getId());
        $this->rental = $this->enquiry->get('Rental');
        $this->setTemplate('confirm');
      }
    }

    public function executePause()
    {
      $rental = $this->getRentalOrCreate();

      $rental->pause();

      $this->getUser()->setFlash('notice', 'Rental has been paused');

      return $this->redirect('rental/index');
    }

    public function executeCopy()
    {
      $rental = $this->getRentalOrCreate();
      $rental->set('owner_id', $this->getUser()->getId());

      $this->form = new OwnerRentalForm(array(), array(
        'country' => $this->getUser()->getCountry(),
        'domain'  => $this->getUser()->getUserDomain(),
        'user'    => $this->getUser()->getAttribute('profile')
      ));

      $rental = $this->form->duplicateFromRental($rental);
      $this->rental = $rental;

      $this->setTemplate('edit');
    }

    public function executeConfirm($request)
    {
      $enquiry = Doctrine::getTable('Enquiry')->find($request->getParameter('id'));

      $this->forward404Unless($enquiry);
      $this->rental = $enquiry->get('Rental');
      $this->enquiry = $enquiry;
    }

    protected function addSortCriteria ($q)
    {
      parent::addSortCriteria($q);

      if ($sort_column = $this->getUser()->getAttribute('sort', null, 'sf_admin/rental/sort'))
      {
        $type = $this->getUser()->getAttribute('type', null, 'sf_admin/rental/sort') == 'asc' ? '' : 'desc';

        switch ($sort_column) {
          case 'Manager':
            $q->orderBy('Rental.ListedBy.firstname '.$type.', Rental.ListedBy.lastname '.$type);
            break;
          case 'view_id':
            $q->orderBy('Rental.id '.$type);
            break;
          case 'VehicleType':
            $q->orderBy('Rental.VehicleType.description '.$type);
            break;
          case 'PickupLocation':
            $q->orderBy('Rental.PickupLocation.name '.$type);
            break;
          case 'DropoffLocation':
            $q->orderBy('Rental.DropoffLocation.name '.$type);
            break;
          case 'date_available_from':
            $q->addSelect('IF(Rental.accepted_request_id, Rental.AcceptedRequest.PickupDate, Rental.DateAvailableFrom) real_pickup_date');
            $q->orderBy('real_pickup_date '.$type);
            break;
        }
      }
    }

    protected function getRentalOrCreate ($id = 'id')
    {
        $rental = parent::getRentalOrCreate($id);

        if ($rental->exists()) {
            $this->forward404Unless($rental->get('owner_id') === $this->getUser()->getOwnerId());
        }

        return $rental;
    }

    protected function deleteRental($rental)
    {
      $rental->remove(true);
    }

    protected function addFiltersCriteria ($q)
    {
        $q->addWhere('Rental.owner_id = ?', $this->getUser()->getOwnerId());

        parent::addFiltersCriteria($q);
    }
}
