<?php

/**
 * request actions.
 *
 * @package    symfony
 * @subpackage request
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 5647 2007-10-23 17:25:20Z Eric.Fredj $
 */
class requestActions extends autorequestActions
{
    public function executeManage(sfWebRequest $request)
    {
        $this->listing = Doctrine::getTable('Listing')->find($request->getParameter('listing_id'));

        $this->forward404Unless($this->listing && // listing exists
                                $this->listing->owner_id == $this->getUser()->getOwnerId() && // owned by the current user
                                $this->listing->isStatus(Listing::STATUS_NEW) // listing must be new
        );

        $this->requests = Doctrine::getTable('Request')->findByListingId($request->getParameter('listing_id'));
    }

    public function executeReject(sfWebRequest $sfRequest)
    {
        $request = $this->getRequestOrCreate();

        $listing = $request->getListing();

        if ($request->get('status') == Request::STATUS_PENDING)
        {
            $request->reject();
            $this->getUser()->setFlash('notice', 'The request has been rejected.');
        }

        $this->redirect('request/manage?listing_id='.$request->get('listing_id'));
    }

    public function executeAccept(sfWebRequest $request)
    {
        $this->request = $this->getRequestOrCreate();
        $this->listing = $this->request->get('Listing');
        $this->forwardUnless($this->request->isStatus(Request::STATUS_PENDING) && $this->listing->isStatus(Listing::STATUS_NEW), 'request', 'error');
        $this->user = $this->getUser()->getAttribute('profile');
        $this->form = new RequestAcceptForm(array(), array(
            'user' => $this->user,
            'price' => $this->listing->getTotalPrice($this->request),
            'country' => $this->getUser()->getAttribute('profile')->getCountryDomainId()
        ));

        if ($request->isMethod('POST'))
        {
            $this->form->bind($request->getParameter('confirm'));

            if ($this->form->isValid())
            {
                if ($this->form->getValue('credits')) {
                    $this->getUser()->setAttribute("payment_info", $this->form->getValue('credits'));
                    $this->getUser()->setAttribute('payment_redirect', 'request/accept?id='.$this->request->getId());
                    $this->redirect("payment/chargeAccount?req_id=".$this->request->getId());
                }

                if ($this->request->accept($this->getUser()->getProfile()->getUsername()))
                {
                    $this->getUser()->getAttribute('profile')->refresh();

                    $this->redirectUnless($request->isXmlHttpRequest(), 'listing/confirm?id='.$this->request->getListingId());

                    $this->form = new ConfirmBookingForm(array(
                      'internal_reference' => $this->listing->get('internal_reference')
                    ));

                    $this->setTemplate('confirm', 'listing');
                }
            }
        }
    }

    public function executeError()
    {
        $request = $this->getRequestOrCreate();

        if ($request && $request->status == Request::STATUS_ACCEPTED) {
            return $this->renderText('You already accepted this driver.');
        }

        if ($request && $request->status == Request::STATUS_REJECTED) {
            return $this->renderText('You already rejected this driver.');
        }

        if ($request && $request->status == Request::STATUS_OTHER_CHOOSE) {
            return $this->renderText('Another owner already accepted this driver.');
        }

        if ($request && $request->status == Request::STATUS_DRIVER_CANCELED) {
            return $this->renderText('Driver cancelled the request.');
        }

        return $this->renderText('You can\'t accept this request');
    }

    public function executeAccepted(sfWebRequest $request)
    {
        $this->request = $this->getRequestOrCreate();
        $this->listing = $this->request->get('Listing');
    }

    public function executeEmailaccept(sfWebRequest $request)
    {
        if ($this->getUser()->isAuthenticated()) {
            $req = $this->getRequestOrCreate();
            return $this->redirect('listing/view?id='.$req->get('listing_id').'&accept='.$req->getId());
        }

        // URL based authentication
        $req = parent::getRequestOrCreate();
        $user = $this->findUser($request->getParameter('uid'));
        if ($user && $req->checkKey($request->getParameter('key'), $user)) {
            // Manager validated, let's authenticate him now.
            $this->getUser()->authenticate($user);
        }

        // redirect to the listing page
        // if not able to authenticate above, let symfony handle authentication
        return $this->redirect('listing/view?id='.$req->get('listing_id').'&accept='.$req->getId());
    }

    public function executeEmailreject(sfWebRequest $request)
    {
        if ($this->getUser()->isAuthenticated()) {
            $req = $this->getRequestOrCreate();
            return $this->redirect('request/reject?id='.$req->get('id'));
        }

        // URL based authentication
        $req = parent::getRequestOrCreate();
        $user = $this->findUser($request->getParameter('uid'));
        if ($user && $req->checkKey($request->getParameter('key'), $user)) {
            // owner authenticated, how to log in him? owner user!!
            $this->getUser()->authenticate($user);
        }

        // not able to authenticate, let symfony handle authentication
        return $this->redirect('request/reject?id='.$req->get('id'));
    }

    protected function findUser($uid)
    {
        return Doctrine::getTable('OwnerUser')->find($uid);
    }

    protected function addFiltersCriteria ($q)
    {
        $q
            ->innerJoin('Request.Listing l')
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->addWhere('l.PickupLocation.Category.country_id = ?', $this->getUser()->getCountry())
            ->addWhere('l.status != ?', Listing::STATUS_DELETED)
        ;

        parent::addFiltersCriteria($q);
    }

    protected function getRequestOrCreate ($id = 'id')
    {
        $request = parent::getRequestOrCreate($id);

        if ($request->exists()) {
            $this->forward404Unless($request->get('Listing')->get('owner_id') === $this->getUser()->getOwnerId());
        }

        $this->setVar('request', $request);

        return $request;
    }

    public function getRequest()
    {
      return $this->context->getRequest();
    }
}
